/*jslint node: true */
var shell = require('shelljs'),
  path = require('path'),
  webpack = require('webpack'),
  argv = require('yargs').argv,
  UglifyJsPlugin = webpack.optimize.UglifyJsPlugin;

var plugins = [];

if (argv.minify === 'true') {
  plugins.push(new UglifyJsPlugin({ minimize: true }));
}

if (argv.prettier === 'true') {
  var PrettierPlugin = require('prettier-webpack-plugin');

  plugins.push(new PrettierPlugin({
    tabWidth: 4,
    singleQuote: true,
  }));
}

module.exports = function (grunt) {
  'use strict';

  // Project configuration.
  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    clean: {
      contents: ['dist/**/*']
    },

    watch: {
      configFiles: {
        files: ['gruntfile.js'],
        options: {
          reload: true
        }
      },

      scss: {
        files: 'src/scss/**/*.scss',
        tasks: ['sass', 'postcss', 'styleguide'],
        options: {
          atBegin: true
        }
      },

      copy: {
        files: ['src/img/**/*', 'src/fonts/**/*', 'src/html/**/*.html'],
        options: {
          atBegin: true
        }
      },
    },

    sass: {
      dist: {
        options: {
          outputStyle: 'nested', // compression is done via postcss
          sourceMap: false,
          includePaths: ['./node_modules', './src/scss']
        },
        files: {
          'dist/css/styles.css': 'src/scss/styles.scss',
          'dist/css/styleguide.css': 'src/scss/styleguide.scss'
        }
      }
    },

    copy: {
      fonts: {
        files: [{
          expand: true,
          cwd: 'src/fonts/',
          src: ['*.woff', '*.eot'],
          dest: 'dist/fonts'
        }]
      },
      images: {
        files: [{
          expand: true,
          cwd: 'src/img/',
          src: ['**'],
          dest: 'dist/img'
        }]
      }
    },

    postcss: {
      options: {
        map: false,
        processors: [
          require('autoprefixer')({
            browsers: 'last 2 versions'
          }), // Add vendor prefixes
          require('cssnano')({
            zindex: false
          }) // minify the result
        ]
      },

      dist: {
        src: 'dist/css/*.css'
      }
    },

    karma: {
      dev: {
        configFile: 'karma.conf.js',
        reporters: ['progress', 'coverage']
      },
      teamcity: {
        configFile: 'karma.conf.js',
        reporters: ['teamcity']
      }
    },
    protractor: {
      options: {
        troubleshoot: true,
        configFile: 'spec/protractor-config.js', // Default config file
        keepAlive: true, // If false, the grunt process stops when the test fails.
        noColor: false, // If true, protractor will not use colors in its output.
        args: {
          // Arguments passed to the command
        }
      },
      all: {}
    },

    express: {
      options: {
        port: 8080,
        debug: true
      },
      server: {
        options: {
          script: 'server.js'
        }
      }
    }
  });

  grunt.registerTask('styleguide', 'Run the styleguide generator', function () {
    shell.exec('styleguide');
  });

  // Load the plugins.
  grunt.loadNpmTasks('grunt-contrib-clean');
  grunt.loadNpmTasks('grunt-contrib-connect');
  grunt.loadNpmTasks('grunt-contrib-copy');
  grunt.loadNpmTasks('grunt-contrib-uglify');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-express-server');
  grunt.loadNpmTasks('grunt-webpack');
  grunt.loadNpmTasks('grunt-karma');
  grunt.loadNpmTasks('grunt-postcss');
  grunt.loadNpmTasks('grunt-protractor-runner');
  grunt.loadNpmTasks('grunt-sass');

  // Default task(s).
  grunt.registerTask('default', ['clean', 'sass', 'postcss', 'copy', 'styleguide', 'express', 'watch']);
  grunt.registerTask('styles', ['sass', 'postcss']);

  grunt.registerTask('acceptance', ['protractor']);
  grunt.registerTask('test', ['sass', 'postcss', 'copy', 'acceptance', 'karma:dev']);
  grunt.registerTask('build', ['clean', 'sass', 'postcss', 'copy', 'styleguide']);

};
